@@ -240,13 +240,14 @@ class LSHAttention(nn.Module):
             rotated_vecs = torch.squeeze(rotated_vecs, 0)
             bucket_range = torch.arange(rotated_vecs.shape[-1], device=device)
             bucket_range = torch.reshape(bucket_range, (1, -1))
-            bucket_range = bucket_range.expand_as(rotated_vecs.shape)
+            bucket_range = bucket_range.expand_as(rotated_vecs)
 
             _, buckets = sort_key_val(rotated_vecs, bucket_range, dim=-1)
-            buckets = buckets[:, -self.n_hashes:]
+            # buckets size [batch size, _rehash_each_round,seq_len, buckets]
+            buckets = buckets[:, :, :, -self.n_hashes:]
 
-            h, *_ = buckets.shape 
-            buckets = torch.reshape(buckets.permute((*_, h)), (-1,))
+            h, *_ = buckets.shape
+            buckets = torch.reshape(buckets, (h, -1))
 
         return buckets
 
@@ -703,6 +704,7 @@ class ReformerLM(nn.Module):
             self.pos_emb = AxialPositionalEmbedding(emb_dim, axial_position_shape)
 
         self.reformer = Reformer(dim, depth, max_seq_len, heads = heads, dim_head = dim_head, bucket_size = bucket_size, n_hashes = n_hashes, ff_chunks = ff_chunks, attn_chunks = attn_chunks, causal = causal, weight_tie = weight_tie, lsh_dropout = lsh_dropout, ff_mult = ff_mult, ff_activation = ff_activation, ff_glu = ff_glu, ff_dropout = ff_dropout, post_attn_dropout = 0., layer_dropout = layer_dropout, random_rotations_per_head = random_rotations_per_head, twin_attention = twin_attention, use_scale_norm = use_scale_norm, use_rezero = use_rezero, use_full_attn = use_full_attn, full_attn_thres = full_attn_thres, reverse_thres = reverse_thres, num_mem_kv = num_mem_kv, one_value_head = one_value_head, n_local_attn_heads = n_local_attn_heads, pkm_layers = pkm_layers, pkm_num_keys = pkm_num_keys)
+        self.norm = nn.LayerNorm(dim)
 
         if return_embeddings:
             self.out = Identity()
@@ -719,4 +721,5 @@ class ReformerLM(nn.Module):
 
         x = self.to_model_dim(x)
         x = self.reformer(x, **kwargs)
+        x = self.norm(x)
         return self.out(x)
